/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 *
 * common.h: Common functions, constants and types.
 *
 * Copyright (C) 2006 Katie Stafford
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/

#ifndef __COMMON_H_
#define __COMMON_H_

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>


typedef char sint8;
typedef unsigned char uint8;
typedef short sint16;
typedef unsigned short uint16;
typedef int sint32;
typedef unsigned int uint32;
typedef long long int sint64;
typedef unsigned long long int uint64;

/* 
   Define SDLTEST to build SDL test version. This is a lot easier than
   screwing with the UI simulator.
*/

#ifdef SDLTEST
#include <SDL/SDL.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdbool.h>

#ifndef MAX_PATH
#define MAX_PATH 1024
#endif

#define PATH "./"

#define PLUGIN_HEADER
#define RB
#define ICODE_ATTR
#define IDATA_ATTR
#define IBSS_ATTR

#if SDL_BYTEORDER == SDL_LIL_ENDIAN
#define ROCKBOX_LITTLE_ENDIAN
#endif

#else /* SDLTEST */


#include "plugin.h"

#define PATH "/.rockbox/rocken/"

struct plugin_api rb_api;
struct plugin_api* rb;

#ifndef SIMULATOR
void *__malloc(unsigned int);
void *__calloc(unsigned int, unsigned int);
void *__realloc(void *, unsigned int);
void __free(void *p);
char* __strdup(const char* str);
void malloc_init(unsigned char *buffer, unsigned int size);

#define malloc __malloc
#define calloc __calloc
#define realloc __realloc
#define strdup __strdup
#define free __free

#endif

#define RB rb_api.

#endif

static inline int randrange(int min, int max) {
    return min + (((unsigned)RB rand())%(max - min));
}

void debugf(const char* fmt, ...);

static inline int _expand(void* pdat, int* pcap, int newcap, int msize) {
    void* dat = *((void**)pdat);
    int cap = *pcap;
    if (!dat && newcap) {
        cap = newcap;
        dat = malloc(cap * msize);
        if (!dat)
            return 0;
        *((void**)pdat) = dat;
        *pcap = newcap;
        return 1;
    }
    if (cap < newcap) {
        cap = (cap * 3) / 2;
        if (cap < newcap) {
            cap = newcap;
        }

        dat = realloc(dat, cap * msize);
        if (!dat) {
            return 0;
        }

        *pcap = cap;
        *((void**)pdat) = dat;
    }
    return 1;
}

/* Expand array _ptr_ so it can contain at least _newcap_ items. */

#define expand(ptr, cap, newcap) _expand(&(ptr), &(cap), newcap, sizeof(*(ptr)))

#define CTOR __attribute__((constructor)) 

/* Declare a variable of type _type_ and initialize it. The
   initializer is allowed to be non-static. */

#define INITVAR(type, name, value)                              \
type name;                                                      \
static void CTOR __construct_##name (void) { name = value; }

#define ASSERT(c,r) if (!(c)) return r

#endif
