/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 *
 * Copyright (C) 2006 Katie Stafford
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/

#ifndef __FILE_H_
#define __FILE_H_

#include "common.h"

typedef struct {
    uint8 *data;
    int len, pos, cap;
} buffer;

enum {
    PHASE_WRITE,
    PHASE_READ,
    PHASE_LINK,
    PHASE_RESET
};

/* Read from a buffer */
uint8 readu8(buffer * buf);
uint16 readu16(buffer * buf);
uint32 readu32(buffer * buf);

uint16 readu16le(buffer * buf);
uint32 readu32le(buffer * buf);
void readstr(buffer * buf, char* str, int l);
int readstr_alloc(buffer * buf, char** str);

#define reads8(buf) ((signed)readu8(buf))
#define reads16(buf) ((signed)readu16(buf))
#define reads32(buf) ((signed)readu32(buf))

#define reads16le(buf) ((signed)readu16le(buf))
#define reads32le(buf) ((signed)readu32le(buf))

void writeu8(buffer* buf, uint8 v);
void writeu16(buffer* buf, uint16 v);
void writeu32(buffer* buf, uint32 v);

void writestr(buffer* buf, const char* str);
void writebuf(buffer* buf, uint8* data, int l);
void writedata(buffer* buf, uint8* data, int l);

void serial_u8(uint8* p, buffer* buf, int phase);
void serial_u16(uint16* p, buffer* buf, int phase);
void serial_u32(uint32* p, buffer* buf, int phase);
void serial_str(char** s, buffer* buf, int phase);
void serial_buffer(buffer* buf2, buffer* buf, int phase);

int loadfile(const char* fname, buffer* buf);
int loadgzfile(const char* fname, buffer* buf);
int writefile(const char* fname, buffer* buf);

#endif
