/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 *
 * Copyright (C) 2006 Katie Stafford
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/

#ifndef __GRAPHICS_H_
#define __GRAPHICS_H_

#include "common.h"
#include "util.h"

/* Size of the game display */
#define XDIM 160
#define YDIM 112

/* 
   Horizontal/vertical ratio. For square aspect, it should be XDIM / 2. 
   If this is set too low, the screen will be squished
   vertically.
*/

#define FOVMUL 340

/* 160*340 >> 7*/
#define XD_FM 425
#define XD_FM_S 7

#define HVAL ((80*FOVMUL)>>8)
#define MAG 3
#define BPL XDIM

#define MAX_SPRITES 512
#define SF_FLIPX 1

typedef enum {
    DIR_WEST,
    DIR_EAST,
    DIR_NORTH,
    DIR_SOUTH,
    DIR_FACE,
    DIR_HIDE
} spritedir;


typedef struct texture {
    uint32 *data;
    uint8 shift;
    uint32 sx, ex;
} texture;

typedef struct {
    char* name;
    int size;
    texture* tex;
} texgroup;

typedef struct {
    int x, y, z;
    int dir, dist;
    int flags;
    texture *tex;
} sprite;

extern texgroup texgrp_board;
extern objectgroup texgroups;

extern sprite sprites[];
extern int num_sprites;

extern unsigned short divmod10x3[];
static inline void putpixel(uint32* d, int s, int x, int y, int v) {
    int shf = divmod10x3[x];
    d += ((shf>>8)<<s) + y;
    *d = (*d & ~(7<<shf)) | v<<shf;
}
static inline int getpixel(uint32* d, int s, int x, int y) {
    int shf = divmod10x3[x];
    d += ((shf>>8)<<s) + y;
    return (*d >> shf) & 7;
}
static inline int picsize(int s) {
    int d = (1<<s);
    return (((d+9)/10)*d)*4;
}

#define TEXGROUP(name)                          \
texgroup texgrp_##name = { #name, 0, NULL };    \
REGISTER_NAMEDOBJECT(texgroups,texgrp_##name)
     
#define TEXGROUP_E(name) extern texgroup texgrp_##name

TEXGROUP_E(board);

int pointseen(int x, int y);
void addsprite_tex(int x, int y, int z, texture* tex, spritedir dir, int flags);
void _addsprite(int x, int y, int z, texgroup* g, int n, spritedir dir, int flags);
#define addsprite(x, y, z, g, n, d, f) _addsprite(x, y, z, &texgrp_##g, n, d, f)


void preparescreen(int posx, int posy, int posz, int ang, int vys, int vye, int vyc) ICODE_ATTR;
void drawscreen(uint8 * screen)  ICODE_ATTR;
void drawsprites(uint8 * screen)  ICODE_ATTR;
int loadtex(const char* fname);

#endif
